@@ -3,10 +3,10 @@
  *
  *	Routines to support keyboard events on the Macintosh.
  *
- * Copyright (c) 1995-1997 Sun Microsystems, Inc.
- * Copyright 2001-2009, Apple Inc.
- * Copyright (c) 2005-2009 Daniel A. Steffen <das@users.sourceforge.net>
- * Copyright (c) 2020 Marc Culler
+ * Copyright © 1995-1997 Sun Microsystems, Inc.
+ * Copyright © 2001-2009 Apple Inc.
+ * Copyright © 2005-2009 Daniel A. Steffen <das@users.sourceforge.net>
+ * Copyright © 2020 Marc Culler
  *
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
@@ -153,9 +153,7 @@ static int	KeyDataToUnicode(UniChar *uniChars, int maxChars,
 {
     (void)notification;
 #ifdef TK_MAC_DEBUG_NOTIFICATIONS
-    TKLog(@"-[%@(%p) %s] %@", [self class], self, _cmd, notification);
-#else
-    (void)notification;
+    TKLog(@"-[%@(%p) %s] %@", [self class], self, sel_getName(_cmd), notification);
 #endif
     keyboardChanged = YES;
     UpdateKeymaps();
@@ -273,9 +271,9 @@ UpdateKeymaps()
         for (virt = 0; virt < 128; virt++) {
 	    MacKeycode macKC;
 	    macKC.v = (keycode_v) {.virt = virt, .o_s = index, .keychar = 0};
-	    int modifiers = INDEX2CARBON(index), result;
+	    int modifiers = INDEX2CARBON(index);
 	    UniChar keychar = 0;
-	    result = KeyDataToUnicode(&keychar, 1, kUCKeyActionDown, virt,
+	    KeyDataToUnicode(&keychar, 1, kUCKeyActionDown, virt,
 				      modifiers, NULL);
 	    if (keychar == 0x10) {
 
